#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "pedigree.h"
#include "rpedigree.h"
#include "readline.h"
#include "dlist.h"

// create tree from names in file
pedigree* read_pedigree(FILE* fp) {
    char* name;
    
    // start dlist_el, read first name
    if (!(name = readline(fp))) {
        return 0;
    }
    pedigree* ret = create_node(name,0,0);
    dlist_el* first = create_dlist_el(ret);
       
    /*
     The following loop walks through the double linked list
     and replaces each element containing a child 
     with two elements containing its parents.
    */
    dlist_el* child = first;
    for (;;) {
        // free old string
        free(name);
        // create and insert father
        if (!(name = readline(fp))) {
            if (child == first) return ret; //OK
            else {
                fprintf(stderr,"Not enough names.\n");
                return 0;
            }
        }

        dlist_el* father = create_dlist_el(create_node(name,0,0));
        
        if (child == first) {
            first = father; // father of first child in dlist_el is new first element
        }
        
        connect(child->prev, father);
        
        // free old string
        free(name);
        // create and insert mother
        if (!(name = readline(fp))) {
            fprintf(stderr,"Not enough names.\n");
            return 0;
        }

        dlist_el* mother = create_dlist_el(create_node(name,0,0));
        connect(father, mother);
        connect(mother, child->next);
        
        // connect nodes in tree
        set_father(child->p, father->p);
        set_mother(child->p, mother->p);
        
        // go to next child and free old one
        free(child);
        child = mother->next;
        if (!child) child = first;
    }
    
    // free list
    for (dlist_el* e = first; e; ) {
        dlist_el* tmp = e;
        free(tmp);
        e = e->next;
    }
    
    return ret;
}